@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2024 NVIDIA Corporation
+    Copyright (c) 2015-2025 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -58,7 +58,7 @@
 #include "uvm_va_block_types.h"
 #include "uvm_linux.h"
 #include "uvm_types.h"
-#include "nv_uvm_types.h"
+#include "nv_uvm_user_types.h"
 #if UVM_IS_CONFIG_HMM() || defined(CONFIG_PCI_P2PDMA)
 #include <linux/memremap.h>
 #endif
@@ -190,35 +190,47 @@ typedef uvm_chunk_size_t uvm_chunk_sizes_mask_t;
 
 typedef struct uvm_pmm_gpu_chunk_suballoc_struct uvm_pmm_gpu_chunk_suballoc_t;
 
-#if UVM_IS_CONFIG_HMM()
-
-typedef struct uvm_pmm_gpu_struct uvm_pmm_gpu_t;
-
+#if UVM_IS_CONFIG_HMM() || defined(NV_MEMORY_DEVICE_COHERENT_PRESENT)
 typedef struct
 {
+    // For g_uvm_global.devmem_ranges
+    struct list_head list_node;
+
+    // Size that was requested when created this region. This may be less than
+    // the size actually allocated by the kernel due to alignment contraints.
+    // Figuring out the required alignment at compile time is difficult due to
+    // unexported macros, so just use the requested size as the search key.
+    unsigned long size;
+
     struct dev_pagemap pagemap;
 } uvm_pmm_gpu_devmem_t;
+#endif
+
+#if UVM_IS_CONFIG_HMM()
+typedef struct uvm_pmm_gpu_struct uvm_pmm_gpu_t;
 
 // Return the GPU chunk for a given device private struct page.
 uvm_gpu_chunk_t *uvm_pmm_devmem_page_to_chunk(struct page *page);
 
+// Return the va_space for a given device private struct page.
+uvm_va_space_t *uvm_pmm_devmem_page_to_va_space(struct page *page);
+
 // Return the GPU id for a given device private struct page.
 uvm_gpu_id_t uvm_pmm_devmem_page_to_gpu_id(struct page *page);
 
 // Return the PFN of the device private struct page for the given GPU chunk.
 unsigned long uvm_pmm_gpu_devmem_get_pfn(uvm_pmm_gpu_t *pmm, uvm_gpu_chunk_t *chunk);
-
 #endif
 
-#if defined(CONFIG_PCI_P2PDMA) && defined(NV_STRUCT_PAGE_HAS_ZONE_DEVICE_DATA)
-#include <linux/pci-p2pdma.h>
+// Allocate and initialise struct page data in the kernel to support HMM.
+NV_STATUS uvm_pmm_devmem_init(uvm_parent_gpu_t *gpu);
+void uvm_pmm_devmem_deinit(uvm_parent_gpu_t *parent_gpu);
 
-void uvm_pmm_gpu_device_p2p_init(uvm_gpu_t *gpu);
-void uvm_pmm_gpu_device_p2p_deinit(uvm_gpu_t *gpu);
-#else
-static inline void uvm_pmm_gpu_device_p2p_init(uvm_gpu_t *gpu) {}
-static inline void uvm_pmm_gpu_device_p2p_deinit(uvm_gpu_t *gpu) {}
-#endif
+void uvm_pmm_gpu_device_p2p_init(uvm_parent_gpu_t *gpu);
+void uvm_pmm_gpu_device_p2p_deinit(uvm_parent_gpu_t *gpu);
+
+// Free unused ZONE_DEVICE pages.
+void uvm_pmm_devmem_exit(void);
 
 struct uvm_gpu_chunk_struct
 {
@@ -252,6 +264,11 @@ struct uvm_gpu_chunk_struct
 
         // This flag indicates an allocated user chunk is referenced by a device
         // private struct page PTE and therefore expects a page_free() callback.
+        // The flag is only for sanity checking since uvm_pmm_gpu_free()
+        // shouldn't be called if Linux has a device private reference to this
+        // chunk and devmem_page_free() should only be called from the Linux
+        // callback if a reference was created.
+        // See uvm_hmm_va_block_service_locked() and fill_dst_pfn() for details.
         //
         // This field is always false in kernel chunks.
         bool is_referenced : 1;
@@ -281,6 +298,9 @@ struct uvm_gpu_chunk_struct
     // The VA block using the chunk, if any.
     // User chunks that are not backed by a VA block are considered to be
     // temporarily pinned and cannot be evicted.
+    // Note that the chunk state is normally UVM_PMM_GPU_CHUNK_STATE_ALLOCATED
+    // but can also be UVM_PMM_GPU_CHUNK_STATE_TEMP_PINNED if an HMM va_block
+    // and device private struct page have a pointer to this chunk.
     //
     // This field is always NULL in kernel chunks.
     uvm_va_block_t *va_block;
@@ -339,6 +359,13 @@ typedef struct uvm_pmm_gpu_struct
         // uvm_pmm_gpu_mark_root_chunk_(un)used().
         struct list_head va_block_unused;
 
+        // List of discarded root GPU chunks, which are still mapped on the GPU.
+        // Chunks on this list are evicted with a lower priority than unused chunks.
+        //
+        // Updated by the VA block code with
+        // uvm_pmm_gpu_mark_root_chunk_discarded().
+        struct list_head va_block_discarded;
+
         // List of root chunks used by VA blocks
         struct list_head va_block_used;
 
@@ -349,10 +376,6 @@ typedef struct uvm_pmm_gpu_struct
         nv_kthread_q_item_t va_block_lazy_free_q_item;
     } root_chunks;
 
-#if UVM_IS_CONFIG_HMM()
-    uvm_pmm_gpu_devmem_t devmem;
-#endif
-
     // Lock protecting PMA allocation, freeing and eviction
     uvm_rw_semaphore_t pma_lock;
 
@@ -417,22 +440,25 @@ static bool uvm_gpu_chunk_is_user(const uvm_gpu_chunk_t *chunk)
 // page containing the chunk's starting address.
 struct page *uvm_gpu_chunk_to_page(uvm_pmm_gpu_t *pmm, uvm_gpu_chunk_t *chunk);
 
+// Return the physical address of the given chunk. The GPU must support
+// coherence, (uvm_parent_gpu_is_coherent() should return true).
+NvU64 uvm_gpu_chunk_to_sys_addr(uvm_pmm_gpu_t *pmm, uvm_gpu_chunk_t *chunk);
+
 // User memory allocator.
 //
 // Allocates num_chunks chunks of size chunk_size in caller-supplied array
 // (chunks).
 //
-// Returned chunks are in the TEMP_PINNED state, requiring a call to either
-// uvm_pmm_gpu_unpin_allocated, uvm_pmm_gpu_unpin_referenced, or
-// uvm_pmm_gpu_free. If a tracker is passed in, all
-// the pending operations on the allocated chunks will be added to it
+// Returned chunks are in the TEMP_PINNED state, requiring a call to
+// uvm_pmm_gpu_unpin_allocated or uvm_pmm_gpu_free. If a tracker is passed in,
+// all the pending operations on the allocated chunks will be added to it
 // guaranteeing that all the entries come from the same GPU as the PMM.
 // Otherwise, when tracker is NULL, all the pending operations will be
 // synchronized before returning to the caller.
 //
 // Each of the allocated chunks list nodes (uvm_gpu_chunk_t::list) can be used
-// by the caller until the chunk is unpinned (uvm_pmm_gpu_unpin_allocated,
-// uvm_pmm_gpu_unpin_referenced) or freed (uvm_pmm_gpu_free). If used, the list
+// by the caller until the chunk is unpinned (uvm_pmm_gpu_unpin_allocated)
+// or freed (uvm_pmm_gpu_free). If used, the list
 // node has to be returned to a valid state before calling either of the APIs.
 //
 // In case of an error, the chunks array is guaranteed to be cleared.
@@ -465,12 +491,6 @@ NV_STATUS uvm_pmm_gpu_alloc_kernel(uvm_pmm_gpu_t *pmm,
 // Can only be used on user memory.
 void uvm_pmm_gpu_unpin_allocated(uvm_pmm_gpu_t *pmm, uvm_gpu_chunk_t *chunk, uvm_va_block_t *va_block);
 
-// Unpin a temporarily pinned chunk, set its reverse map to a VA block, and
-// mark it as referenced.
-//
-// Can only be used on user memory.
-void uvm_pmm_gpu_unpin_referenced(uvm_pmm_gpu_t *pmm, uvm_gpu_chunk_t *chunk, uvm_va_block_t *va_block);
-
 // Free a user or kernel chunk. Temporarily pinned chunks are unpinned.
 //
 // The tracker is optional and a NULL tracker indicates that no new operation
@@ -548,6 +568,9 @@ void uvm_pmm_gpu_mark_root_chunk_used(uvm_pmm_gpu_t *pmm, uvm_gpu_chunk_t *chunk
 // Mark an allocated user chunk as unused
 void uvm_pmm_gpu_mark_root_chunk_unused(uvm_pmm_gpu_t *pmm, uvm_gpu_chunk_t *chunk);
 
+// Mark an allocated chunk as discarded
+void uvm_pmm_gpu_mark_root_chunk_discarded(uvm_pmm_gpu_t *pmm, uvm_gpu_chunk_t *chunk);
+
 static bool uvm_gpu_chunk_same_root(uvm_gpu_chunk_t *chunk1, uvm_gpu_chunk_t *chunk2)
 {
     return UVM_ALIGN_DOWN(chunk1->address, UVM_CHUNK_SIZE_MAX) == UVM_ALIGN_DOWN(chunk2->address, UVM_CHUNK_SIZE_MAX);
