@@ -35,48 +35,245 @@
 
 using namespace DisplayPort;
 
-void DPCDHALImpl2x::performCableIdHandshake()
+void DPCDHALImpl2x::parseAndSetCableId(NvU8 cableId)
 {
-    NvU8 byte = 0;
+    caps2x.rxCableCaps.bUHBR_10GSupported = true;
 
-    if (AuxRetry::ack ==
-        bus.read(NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX, &byte, sizeof byte))
+    caps2x.rxCableCaps.bUHBR_20GSupported =
+        FLD_TEST_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPRX, _UHBR20_10_CAPABILITY, _10_AND_20_GBPS_SUPPORTED, cableId);
+
+    caps2x.rxCableCaps.bUHBR_13_5GSupported =
+        FLD_TEST_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPRX, _13_5_GBPS_SUPPORTED, _YES, cableId);
+
+    switch (DRF_VAL(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPRX, _CABLE_TYPE, cableId))
     {
-        caps2x.cableCaps.bUHBR_10GSupported =
-            FLD_TEST_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPRX, _UHBR20_10_CAPABILITY, _10_0_GBPS_SUPPORTED, byte) ||
-            FLD_TEST_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPRX, _UHBR20_10_CAPABILITY, _10_AND_20_GBPS_SUPPORTED, byte);
+        case NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX_CABLE_TYPE_CABLE_TYPE_UNKNOWN:
+            caps2x.rxCableCaps.cableType = CableTypeUnknown;
+            break;
+        case NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX_CABLE_TYPE_PASSIVE:
+            caps2x.rxCableCaps.cableType = CableTypePassive;
+            break;
+        case NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX_CABLE_TYPE_LRD:
+            caps2x.rxCableCaps.cableType = CableTypeLRD;
+            break;
+        case NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX_CABLE_TYPE_ACTIVE_RETIMER:
+            caps2x.rxCableCaps.cableType = CableTypeActiveReTimer;
+            break;
+        default:
+            DP_PRINTF(DP_ERROR, "Unknown cable type\n");
+            break;
+    }
+}
 
-        caps2x.cableCaps.bUHBR_20GSupported =
-            FLD_TEST_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPRX, _UHBR20_10_CAPABILITY, _10_AND_20_GBPS_SUPPORTED, byte);
+void DPCDHALImpl2x::performCableIdHandshakeForTypeC()
+{
+    NvU8 txCableCaps = 0;
+    NvU8 rxCableCaps = 0;
 
-        caps2x.cableCaps.bUHBR_13_5GSupported =
-            FLD_TEST_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPRX, _13_5_GBPS_SUPPORTED, _YES, byte);
+    if (AuxRetry::ack !=
+        bus.read(NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX, &rxCableCaps, sizeof rxCableCaps))
+    {
+        DP_PRINTF(DP_WARNING, "Failed to read NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX for updated results");
+    }
+    else
+    {
+        parseAndSetCableId(rxCableCaps);
+    }
 
-        switch (DRF_VAL(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPRX, _CABLE_TYPE, byte))
+    if (caps2x.txCableCaps.bIsSupported)
+    {
+        if (caps2x.txCableCaps.bUHBR_20GSupported)
         {
-            case NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX_CABLE_TYPE_CABLE_TYPE_UNKNOWN:
-                caps2x.cableCaps.cableType = CableTypeUnknown;
+            txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _UHBR20_10_CAPABILITY,
+                                      _10_AND_20_GBPS_SUPPORTED, txCableCaps);
+        }
+        else
+        {
+            txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _UHBR20_10_CAPABILITY,
+                                      _10_0_GBPS_SUPPORTED, txCableCaps);
+        }
+        if (caps2x.txCableCaps.bUHBR_13_5GSupported)
+        {
+            txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _13_5_GBPS_SUPPORTED,
+                                      _YES, txCableCaps);
+        }
+        switch (caps2x.txCableCaps.cableType)
+        {
+            case CableTypePassive:
+                txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _CABLE_TYPE, _PASSIVE, txCableCaps);
                 break;
-            case NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX_CABLE_TYPE_PASSIVE:
-                caps2x.cableCaps.cableType = CableTypePassive;
+            case CableTypeActiveReTimer:
+                txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _CABLE_TYPE, _ACTIVE_RETIMER, txCableCaps);
                 break;
-            case NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX_CABLE_TYPE_LRD:
-                caps2x.cableCaps.cableType = CableTypeLRD;
-                break;
-            case NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX_CABLE_TYPE_ACTIVE_RETIMER:
-                caps2x.cableCaps.cableType = CableTypeActiveReTimer;
+            case CableTypeLRD:
+                txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _CABLE_TYPE, _LRD, txCableCaps);
                 break;
+            case CableTypeOptical:
+            case CableTypeUnknown:
             default:
-                DP_PRINTF(DP_ERROR, "Unknown cable type\n");
+                txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _CABLE_TYPE, _CABLE_TYPE_UNKNOWN, txCableCaps);
                 break;
         }
     }
-
-    if (byte == 0)
+    else if (bCableVconnSourceUnknown)
     {
-        caps2x.cableCaps.bUHBR_10GSupported = true;
+        if (caps2x.bUHBR_20GSupported)
+        {
+            txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _UHBR20_10_CAPABILITY,
+                                      _10_AND_20_GBPS_SUPPORTED, txCableCaps);
+        }
+        else
+        {
+            txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _UHBR20_10_CAPABILITY,
+                                      _10_0_GBPS_SUPPORTED, txCableCaps);
+        }
+        if (caps2x.bUHBR_13_5GSupported)
+        {
+            txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _13_5_GBPS_SUPPORTED,
+                                      _YES, txCableCaps);
+        }
+
+        // Set cable type based on if any LTTPR is detected.
+        if (!bLttprSupported || (this->caps.phyRepeaterCount == 0))
+        {
+            txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _CABLE_TYPE,
+                                      _PASSIVE, txCableCaps);
+        }
+        else
+        {
+            txCableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _CABLE_TYPE,
+                                      _ACTIVE_RETIMER, txCableCaps);
+        }
+    }
+
+    if (caps2x.txCableCaps.bIsSupported)
+    {
+        NvU8 cableCaps = txCableCaps;
+
+        if (caps2x.txCableCaps.bVconnSource)
+        {
+            // write the least common denominator of 0x2217 read value (rxCableCaps) and the eMarker content (txCableCaps)
+            if (caps2x.txCableCaps.bUHBR_20GSupported && caps2x.rxCableCaps.bUHBR_20GSupported)
+            {
+                cableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _UHBR20_10_CAPABILITY,
+                                        _10_AND_20_GBPS_SUPPORTED, cableCaps);
+            }
+            else if (caps2x.txCableCaps.bUHBR_10GSupported && caps2x.rxCableCaps.bUHBR_10GSupported)
+            {
+                cableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _UHBR20_10_CAPABILITY,
+                                        _10_0_GBPS_SUPPORTED, cableCaps);
+            }
+            else
+            {
+                cableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _UHBR20_10_CAPABILITY,
+                                        _UHBR_NOT_CAPABLE, cableCaps);
+            }
+
+            if (caps2x.txCableCaps.bUHBR_13_5GSupported && caps2x.rxCableCaps.bUHBR_13_5GSupported)
+            {
+                cableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _13_5_GBPS_SUPPORTED,
+                                        _YES, cableCaps);
+            }
+            else
+            {
+                cableCaps = FLD_SET_DRF(_DPCD20, _CABLE_ATTRIBUTES_UPDATED_BY_DPTX, _13_5_GBPS_SUPPORTED,
+                                        _NO, cableCaps);
+            }
+        }
+        else
+        {
+            cableCaps = rxCableCaps;
+        }
+
+        if (AuxRetry::ack != bus.write(NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPTX, &cableCaps,
+                                    sizeof cableCaps))
+        {
+            DP_PRINTF(DP_WARNING, "Failed to write NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPTX");
+        }
+    }
+    else if (bCableVconnSourceUnknown)
+    {
+        // Write sink caps to NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPTX first.
+        if (AuxRetry::ack != bus.write(NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPTX, &txCableCaps,
+                                    sizeof txCableCaps))
+        {
+            DP_PRINTF(DP_WARNING, "Failed to write NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPTX");
+        }
+
+        // Check Cable ID from DPRX
+        if (AuxRetry::ack !=
+            bus.read(NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX, &rxCableCaps, sizeof rxCableCaps))
+        {
+            DP_PRINTF(DP_WARNING, "Failed to read NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX for updated results");
+        }
+        else
+        {
+            parseAndSetCableId(rxCableCaps);
+        }
+
+        // If no matches, reflect that to the DPRX
+        if (txCableCaps != rxCableCaps)
+        {
+            if (AuxRetry::ack !=
+                bus.write(NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPTX, &rxCableCaps, sizeof rxCableCaps))
+            {
+                DP_PRINTF(DP_WARNING, "Failed to update NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPTX -> %02x", rxCableCaps);
+            }
+        }
+    }
+}
+
+void DPCDHALImpl2x::performCableIdHandshake()
+{
+    NvU8 rxCableCaps    = 0;
+
+    if (this->bConnectorIsTypeC)
+    {
+        return performCableIdHandshakeForTypeC();
+    }
+
+    // Read Cable ID from DPRX
+    if (AuxRetry::ack ==
+        bus.read(NV_DPCD20_CABLE_ATTRIBUTES_UPDATED_BY_DPRX, &rxCableCaps, sizeof rxCableCaps))
+    {
+        parseAndSetCableId(rxCableCaps);
+    }
+}
+
+void DPCDHALImpl2x::setUSBCCableIDInfo(NV0073_CTRL_DP_USBC_CABLEID_INFO *cableIDInfo)
+{
+    if (!cableIDInfo)
+    {
+        // Reset state as there is no limitation to be imposed from Tx Cable Info
+        resetTxCableCaps();
         return;
     }
+
+    caps2x.txCableCaps.bIsSupported           = true;
+    caps2x.txCableCaps.bUHBR_10GSupported     = cableIDInfo->uhbr10_0_capable;
+    caps2x.txCableCaps.bUHBR_13_5GSupported   = cableIDInfo->uhbr13_5_capable;
+    caps2x.txCableCaps.bUHBR_20GSupported     = cableIDInfo->uhbr20_0_capable;
+    caps2x.txCableCaps.bVconnSource           = cableIDInfo->vconn_source;
+    switch (cableIDInfo->type)
+    {
+        case NV0073_CTRL_DP_USBC_CABLEID_CABLETYPE_PASSIVE:
+            caps2x.txCableCaps.cableType = CableTypePassive;
+            break;
+        case NV0073_CTRL_DP_USBC_CABLEID_CABLETYPE_ACTIVE_RETIMER:
+            caps2x.txCableCaps.cableType = CableTypeActiveReTimer;
+            break;
+        case NV0073_CTRL_DP_USBC_CABLEID_CABLETYPE_ACTIVE_LIN_REDRIVER:
+            caps2x.txCableCaps.cableType = CableTypeLRD;
+            break;
+        case NV0073_CTRL_DP_USBC_CABLEID_CABLETYPE_OPTICAL:
+            caps2x.txCableCaps.cableType = CableTypeOptical;
+            break;
+        case NV0073_CTRL_DP_USBC_CABLEID_CABLETYPE_UNKNOWN:
+        default:
+            caps2x.txCableCaps.cableType = CableTypeUnknown;
+            break;
+    }
+    return;
 }
 
 void DPCDHALImpl2x::parseAndReadCaps()
@@ -88,6 +285,16 @@ void DPCDHALImpl2x::parseAndReadCaps()
 
     DPCDHALImpl::parseAndReadCaps();
 
+    // reset DP tunneling UHBR caps
+    caps2x.dpInTunnelingCaps.bUHBR_10GSupported = NV_FALSE;
+    caps2x.dpInTunnelingCaps.bUHBR_13_5GSupported = NV_FALSE;
+    caps2x.dpInTunnelingCaps.bUHBR_20GSupported = NV_FALSE;
+
+    // reset rxCableCaps
+    caps2x.rxCableCaps.bUHBR_10GSupported = NV_TRUE;
+    caps2x.rxCableCaps.bUHBR_13_5GSupported = NV_TRUE;
+    caps2x.rxCableCaps.bUHBR_20GSupported = NV_TRUE;
+
     // 02206h
     if (AuxRetry::ack == bus.read(NV_DPCD14_EXTENDED_MAIN_LINK_CHANNEL_CODING, &buffer[0], 1))
     {
@@ -95,7 +302,8 @@ void DPCDHALImpl2x::parseAndReadCaps()
                                                           _ANSI_128B_132B, _YES,
                                                           buffer[0]);
 
-        if (caps2x.bDP2xChannelCodingSupported == true)
+        // Read this unconditionally when the connection is tunneled
+        if (caps2x.bDP2xChannelCodingSupported == true || caps.dpInTunnelingCaps.bIsSupported)
         {
             // 0x2215
             if (AuxRetry::ack == bus.read(NV_DPCD20_128B_132B_SUPPORTED_LINK_RATES, &buffer[0], 1))
@@ -183,7 +391,7 @@ void DPCDHALImpl2x::parseAndReadCaps()
         }
     }
 
-    if (!bIgnoreCableIdCaps)
+    if (caps2x.bDP2xChannelCodingSupported && !bIgnoreCableIdCaps)
     {
         performCableIdHandshake();
     }
@@ -201,7 +409,7 @@ void DPCDHALImpl2x::parseAndReadCaps()
     if (caps2x.dpInTunnelingCaps.bDP2xChannelCodingSupported)
     {
         if (AuxRetry::ack ==
-            bus.read(NV_DPCD20_DP_TUNNELING_MAIN_LINK_CHANNEL_CODING, &byte, sizeof byte))
+            bus.read(NV_DPCD20_DP_TUNNELING_128B132B_LINK_RATES, &byte, sizeof byte))
         {
             caps2x.dpInTunnelingCaps.bUHBR_10GSupported =
                 FLD_TEST_DRF(_DPCD20, _DP_TUNNELING_128B132B_LINK_RATES, _10_0_GPBS_SUPPORTED, _YES, byte);
@@ -231,6 +439,11 @@ void DPCDHALImpl2x::setGpuDPSupportedVersions(NvU32 _gpuDPSupportedVersions)
     return DPCDHALImpl::setGpuDPSupportedVersions(_gpuDPSupportedVersions);
 }
 
+void DPCDHALImpl2x::setConnectorTypeC(bool bTypeC)
+{
+    this->bConnectorIsTypeC = bTypeC;
+}
+
 AuxRetry::status DPCDHALImpl2x::notifySDPErrDetectionCapability()
 {
     NvU8 config = 0;
@@ -274,12 +487,18 @@ AuxRetry::status DPCDHALImpl2x::notifySDPErrDetectionCapability()
 bool DPCDHALImpl2x::isDp2xChannelCodingCapable()
 {
     // return false if the device does not support 128b/132b.
-    if (!caps2x.bDP2xChannelCodingSupported)
-        return false;
-
-    // return false if DP-IN Tunneling is supported but not support 128b/132b.
-    if (caps.dpInTunnelingCaps.bIsSupported && !caps2x.dpInTunnelingCaps.bDP2xChannelCodingSupported)
-        return false;
+    // However when dpTunneling is enabled, read the tunneling cap instead
+    if (caps.dpInTunnelingCaps.bIsSupported)
+    {
+        // return false if DP-IN Tunneling is supported but not support 128b/132b.
+        if (!caps2x.dpInTunnelingCaps.bDP2xChannelCodingSupported)
+            return false;
+    }
+    else
+    {
+        if (!caps2x.bDP2xChannelCodingSupported)
+            return false;
+    }
 
     // return true if there is no LTTPR.
     if (!bLttprSupported || (caps.phyRepeaterCount == 0))
@@ -342,11 +561,22 @@ NvU32 DPCDHALImpl2x::getUHBRSupported()
     bool    bUHBR_13_5GSupported = caps2x.bUHBR_13_5GSupported;
     bool    bUHBR_20GSupported   = caps2x.bUHBR_20GSupported;
 
+    // When tunneling is supported and bw allocation is enabled, override the caps from tunneling caps
+    if (caps.dpInTunnelingCaps.bIsSupported && bIsDpTunnelBwAllocationEnabled)
+    {
+        bUHBR_10GSupported   = caps2x.dpInTunnelingCaps.bUHBR_10GSupported;
+        bUHBR_13_5GSupported = caps2x.dpInTunnelingCaps.bUHBR_13_5GSupported;
+        bUHBR_20GSupported   = caps2x.dpInTunnelingCaps.bUHBR_20GSupported;
+    }
+
     if (!bIgnoreCableIdCaps)
     {
-        bUHBR_10GSupported   = bUHBR_10GSupported   && caps2x.cableCaps.bUHBR_10GSupported;
-        bUHBR_13_5GSupported = bUHBR_13_5GSupported && caps2x.cableCaps.bUHBR_13_5GSupported;
-        bUHBR_20GSupported   = bUHBR_20GSupported   && caps2x.cableCaps.bUHBR_20GSupported;
+        bUHBR_10GSupported   = bUHBR_10GSupported   &&  caps2x.rxCableCaps.bUHBR_10GSupported
+                                                            && caps2x.txCableCaps.bUHBR_10GSupported;
+        bUHBR_13_5GSupported = bUHBR_13_5GSupported &&  caps2x.rxCableCaps.bUHBR_13_5GSupported
+                                                            && caps2x.txCableCaps.bUHBR_13_5GSupported;
+        bUHBR_20GSupported   = bUHBR_20GSupported   &&  caps2x.rxCableCaps.bUHBR_20GSupported
+                                                            && caps2x.txCableCaps.bUHBR_20GSupported;
     }
 
     if (caps.phyRepeaterCount > 0)
@@ -356,13 +586,6 @@ NvU32 DPCDHALImpl2x::getUHBRSupported()
         bUHBR_20GSupported   = bUHBR_20GSupported   && caps2x.repeaterCaps.bUHBR_20GSupported;
     }
 
-    if (caps.dpInTunnelingCaps.bIsSupported && bIsDpTunnelBwAllocationEnabled)
-    {
-        bUHBR_10GSupported   = bUHBR_10GSupported   && caps2x.dpInTunnelingCaps.bUHBR_10GSupported;
-        bUHBR_13_5GSupported = bUHBR_13_5GSupported && caps2x.dpInTunnelingCaps.bUHBR_13_5GSupported;
-        bUHBR_20GSupported   = bUHBR_20GSupported   && caps2x.dpInTunnelingCaps.bUHBR_20GSupported;
-    }
-
     if (bUHBR_10GSupported)
     {
         uhbrCaps = FLD_SET_DRF(0073_CTRL_CMD_DP, _GET_CAPS_UHBR_SUPPORTED, _UHBR10_0, _YES, uhbrCaps);
@@ -379,6 +602,24 @@ NvU32 DPCDHALImpl2x::getUHBRSupported()
     return uhbrCaps;
 }
 
+void DPCDHALImpl2x::overrideCableIdCap(LinkRate linkRate, bool bEnable)
+{
+    switch (linkRate)
+    {
+        case dp2LinkRate_20_0Gbps:
+            caps2x.rxCableCaps.bUHBR_20GSupported = bEnable;
+            break;
+        case dp2LinkRate_13_5Gbps:
+            caps2x.rxCableCaps.bUHBR_13_5GSupported = bEnable;
+            break;
+        case dp2LinkRate_10_0Gbps:
+            caps2x.rxCableCaps .bUHBR_10GSupported = bEnable;
+            break;
+        default:
+            DP_PRINTF(DP_ERROR, "DPHAL> Invalid link rate (%" LinkRate_fmtu ") to override.", linkRate);
+    }
+}
+
 bool DPCDHALImpl2x::parseTestRequestPhy()
 {
     NvU8 buffer[2] = {0};
