@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2020-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2020-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -263,7 +263,7 @@ knvlinkIsPresent_IMPL
     NV_STATUS status = NV_OK;
 
     // Mark NVLINK as absent when HCC SPT is enabled
-    if (gpuIsCCFeatureEnabled(pGpu) && !gpuIsCCMultiGpuProtectedPcieModeEnabled(pGpu))
+    if (gpuIsCCFeatureEnabled(pGpu) && (!gpuIsCCMultiGpuProtectedPcieModeEnabled(pGpu) && !gpuIsCCMultiGpuNvleModeEnabled(pGpu)))
         return NV_FALSE;
 
     // On GSP clients, retrieve all device discovery info from GSP through RPC
@@ -515,7 +515,7 @@ knvlinkStateLoad_IMPL
     knvlinkFilterBridgeLinks_HAL(pGpu, pKernelNvlink);
 
     // Disconnected links mask tracks links whose remote ends are not discovered
-    pKernelNvlink->disconnectedLinkMask = KNVLINK_GET_MASK(pKernelNvlink, enabledLinks, 32);
+    pKernelNvlink->disconnectedLinkMask = KNVLINK_GET_MASK(pKernelNvlink, enabledLinks, 64);
 
     if (!IS_RTLSIM(pGpu) || pKernelNvlink->bForceEnableCoreLibRtlsims)
     {
@@ -689,6 +689,8 @@ knvlinkStateLoad_IMPL
 
     knvlinkDumpCallbackRegister_HAL(pGpu, pKernelNvlink);
 
+    knvlinkPostSchedulingEnableCallbackRegister_HAL(pGpu, pKernelNvlink);
+
     // Update list of HSHUB supported RBM modes
     knvlinkGetHshubSupportedRbmModes_HAL(pGpu, pKernelNvlink);
 
@@ -794,6 +796,16 @@ knvlinkStatePostLoad_IMPL
         }
     }
 
+    // We check for NVLE enablement here for the production/non-MODS case
+    if (pKernelNvlink->bNvleModeRegkey)
+    {
+        if (!knvlinkIsNvleEnabled_HAL(pGpu, pKernelNvlink))
+        {
+            NV_PRINTF(LEVEL_ERROR," NVLE not enabled on GPU%d\n", pGpu->gpuInstance);
+            return NV_ERR_FEATURE_NOT_ENABLED;
+        }
+    }
+
     status = knvlinkStatePostLoadHal_HAL(pGpu, pKernelNvlink);
     if (status != NV_OK)
     {
@@ -801,6 +813,11 @@ knvlinkStatePostLoad_IMPL
         return status;
     }
 
+    if (!GPU_IS_NVSWITCH_DETECTED(pGpu))
+    {
+        NV_ASSERT_OK_OR_RETURN(knvlinkSetDirectConnectBaseAddress_HAL(pGpu, pKernelNvlink));
+    }
+
     return NV_OK;
 }
 
@@ -830,6 +847,8 @@ knvlinkStateUnload_IMPL
         kbusDestroyFla_HAL(pGpu, GPU_GET_KERNEL_BUS(pGpu));
     }
 
+    knvlinkPostSchedulingEnableCallbackUnregister_HAL(pGpu, pKernelNvlink);
+
     return NV_OK;
 }
 
@@ -959,7 +978,7 @@ knvlinkStatePostUnload_IMPL
         {
             KernelNvlink *pRemoteKernelNvlink = GPU_GET_KERNEL_NVLINK(pRemoteGpu);
 
-            pRemoteKernelNvlink->disconnectedLinkMask |= NVBIT(pKernelNvlink->nvlinkLinks[linkId].remoteEndInfo.linkNumber);
+            pRemoteKernelNvlink->disconnectedLinkMask |= NVBIT64(pKernelNvlink->nvlinkLinks[linkId].remoteEndInfo.linkNumber);
         }
     }
     FOR_EACH_INDEX_IN_MASK_END;
@@ -1102,41 +1121,14 @@ knvlinkShutdownLinks_WORKITEM
 {
     OBJGPU *pGpu                = gpumgrGetGpu(gpuInstance);
     KernelNvlink *pKernelNvlink = GPU_GET_KERNEL_NVLINK(pGpu);
-    NV_STATUS status;
 
     // Sanity Checks
     NV_CHECK_OR_RETURN_VOID(LEVEL_INFO,
             (pGpu != NULL &&
              pKernelNvlink != NULL));
 
-    if (pKernelNvlink->ipVerNvlink >= NVLINK_VERSION_50)
-    {
-        // Unilaterally shut down all enabled links
-        NV2080_CTRL_NVLINK_PROCESS_INIT_DISABLED_LINKS_PARAMS disableLinksParams = { 0 };
-        disableLinksParams.initDisabledLinksMask = pKernelNvlink->enabledLinks;
-        status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                        NV2080_CTRL_CMD_NVLINK_PROCESS_INIT_DISABLED_LINKS,
-                        (void *)&disableLinksParams,
-                        sizeof(NV2080_CTRL_NVLINK_PROCESS_INIT_DISABLED_LINKS_PARAMS));
-
-        if (status != NV_OK)
-        {
-            NV_PRINTF(LEVEL_ERROR, "Failed to send disable link RPC on degraded GPU%d\n",
-                pGpu->gpuInstance);
-        }
-
-        // Suspend and invalidate probe on degraded GPU.
-        if (gpuFabricProbeIsSupported(pGpu))
-        {
-            gpuFabricProbeSuspend(pGpu->pGpuFabricProbeInfoKernel);
-            gpuFabricProbeInvalidate(pGpu->pGpuFabricProbeInfoKernel);
-        }
-    }
-    else
-    {
-        NV_ASSERT_OR_RETURN_VOID(
-            knvlinkCoreShutdownDeviceLinks(pGpu, pKernelNvlink, NV_TRUE) == NV_OK);
-    }
+    NV_ASSERT_OR_RETURN_VOID(
+        knvlinkCoreShutdownDeviceLinks(pGpu, pKernelNvlink, NV_TRUE) == NV_OK);
 }
 
 /*!
@@ -1157,8 +1149,6 @@ knvlinkSetDegradedMode_IMPL
     NvS32         linkId
 )
 {
-    NvU32 gpuInstance;
-    NvBool bShutdownLinks = NV_FALSE;
     NV_STATUS status = NV_OK;
 
     // Prevent invalid shift later
@@ -1178,65 +1168,21 @@ knvlinkSetDegradedMode_IMPL
         return;
     }
 
-    // Find the remote GPU/NVLink attached to this link, if any
-    for (gpuInstance = 0; gpuInstance < NV_MAX_DEVICES; gpuInstance++)
-    {
-        OBJGPU *pRemoteGpu = gpumgrGetGpu(gpuInstance);
-        KernelNvlink *pRemoteKernelNvlink;
-        NvU32 linkMask = (linkId == -1) ? pKernelNvlink->enabledLinks : NVBIT32(linkId);
+    pKernelNvlink->bIsGpuDegraded = NV_TRUE;
+    NV_PRINTF(LEVEL_ERROR,
+            "GPU%d marked Degraded. Error originated on linkId %d!\n",
+            pGpu->gpuInstance, linkId);
 
-        if ((pKernelNvlink->peerLinkMasks[gpuInstance] & linkMask) == 0x0)
-            continue;
+    // Queue a workitem to handle rest of the degraded mode handling
 
-        // Sanity checks
-        if (pRemoteGpu == NULL)
-        {
-            NV_PRINTF(LEVEL_ERROR,
-                    "Failed to get Remote GPU info for linkId %d to update Degraded GPU%d status\n",
-                    linkId, pGpu->gpuInstance);
-
-            continue;
-        }
-
-        pRemoteKernelNvlink = GPU_GET_KERNEL_NVLINK(pRemoteGpu);
-        if (pRemoteKernelNvlink == NULL)
-        {
-            NV_PRINTF(LEVEL_ERROR,
-                    "Failed to get Remote Nvlink info for linkId %d to update Degraded GPU%d status\n",
-                    linkId, pGpu->gpuInstance);
-
-            continue;
-        }
-
-        //
-        // Only mark the current GPU if the peer GPU is not degraded.
-        // In the case where the peer GPU is already degraded then there is no additional
-        // loss of connectivity in the fabric due to the link error that caused this
-        // path to be triggered.
-        //
-        if (!pRemoteKernelNvlink->bIsGpuDegraded)
-        {
-            pKernelNvlink->bIsGpuDegraded = NV_TRUE;
-            bShutdownLinks = NV_TRUE;
-            NV_PRINTF(LEVEL_ERROR,
-                    "GPU%d marked Degraded. Error originated on linkId %d!\n",
-                    pGpu->gpuInstance, linkId);
-
-
-        }
-    }
-
-    if (bShutdownLinks)
-    {
-        // Queue a workitem to handle rest of the degraded mode handling
-
-        NV_CHECK_OK_OR_ELSE(status, LEVEL_ERROR,
-            osQueueWorkItemWithFlags(pGpu, knvlinkShutdownLinks_WORKITEM,
-                                     NULL,
-                                     (OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RO |
-                                      OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE)),
-            return;);
-    }
+    NV_CHECK_OK_OR_ELSE(status,
+        LEVEL_ERROR,
+        osQueueWorkItem(pGpu,
+                        knvlinkShutdownLinks_WORKITEM,
+                        NULL,
+                        (OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RO |
+                         OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE)),
+        return;);
 }
 
 /*!
@@ -1319,26 +1265,26 @@ _knvlinkProcessSysmemLinks
 {
     NV_STATUS status = NV_OK;
 
-#if defined(NVCPU_PPC64LE) || defined(NVCPU_AARCH64)
+#if defined(NVCPU_AARCH64)
     if (pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_SYSMEM_SUPPORT_ENABLED))
     {
         //
-        // In case of IBM or Tegra, the sysmem links will already have
+        // In case of Tegra, the sysmem links will already have
         // been registered in nvlink core library. In order to trigger
         // topology detection, call knvlinkCoreGetRemoteDeviceInfo
         //
         if (!knvlinkIsForcedConfig(pGpu, pKernelNvlink) && !pKernelNvlink->pLinkConnection)
         {
             //
-            // Establish the current link topology and enable IBM CPU/SYSMEM links.
-            // If any of the discovered links are CPU/SYSMEM, they will be trained,
+            // Establish the current link topology and enable SYSMEM links.
+            // If any of the discovered links are SYSMEM, they will be trained,
             // post-enabled, and then enabled in HSHUB when the call has completed.
             //
             status = knvlinkCoreGetRemoteDeviceInfo(pGpu, pKernelNvlink);
             if (status != NV_OK)
             {
                 NV_PRINTF(LEVEL_ERROR,
-                          "Failed call to get remote device info during IBM CPU/SYSMEM links "
+                          "Failed call to get remote device info during SYSMEM links "
                           "setup, failing NVLink StateLoad on GPU%d!!!\n\n",
                           pGpu->gpuInstance);
 
