@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -125,6 +125,12 @@ _krcInitRegistryOverrides
     {
         dword = NV_REG_STR_RM_BREAK_ON_RC_DEFAULT;
     }
+    else
+    {
+        NV_PRINTF(LEVEL_INFO,
+            "BreakOnRc set by regkey " NV_REG_STR_RM_BREAK_ON_RC "= 0x%08x\n",
+            dword);
+    }
 
     pKernelRc->bBreakOnRc = (dword == NV_REG_STR_RM_BREAK_ON_RC_ENABLE);
 
@@ -132,8 +138,11 @@ _krcInitRegistryOverrides
     if (DRF_VAL(_DEBUG, _BREAK_FLAGS, _RC, SYS_GET_INSTANCE()->debugFlags) ==
         NV_DEBUG_BREAK_FLAGS_RC_ENABLE)
     {
+        NV_PRINTF(LEVEL_INFO,
+                  "BreakOnRc overridden by NV_DEBUG_BREAK_FLAGS_RC\n");
         pKernelRc->bBreakOnRc = NV_TRUE;
     }
+    NV_PRINTF(LEVEL_INFO, "BreakOnRc = %d\n", pKernelRc->bBreakOnRc);
 
     if (osReadRegistryDword(pGpu,
                             NV_REG_STR_RM_WATCHDOG_TIMEOUT,
@@ -451,9 +460,8 @@ krcCheckBusError_KERNEL
     NvU32             clDevCtrlStatus          = 0;
     PcieAerCapability clAer;
 
-
     // PCI-E provides extended error reporting
-    if (pKernelBif == NULL || kbifGetBusIntfType_HAL(pKernelBif) !=
+    if (pKernelBif == NULL || gpuGetBusIntfType_HAL(pGpu) !=
                                   NV2080_CTRL_BUS_INFO_TYPE_PCI_EXPRESS)
     {
         return NV_OK;
@@ -470,8 +478,8 @@ krcCheckBusError_KERNEL
                                 &clDevCtrlStatus) == NV_OK &&
         clDevCtrlStatusFlags != 0)
     {
-        NV_PRINTF(LEVEL_ERROR,
-            "PCI-E corelogic status has pending errors (CL_PCIE_DEV_CTRL_STATUS = %08X):\n",
+        NV_PRINTF(LEVEL_INFO,
+            "PCI-E corelogic: Pending errors in DEV_CTRL_STATUS = %08X\n",
             clDevCtrlStatus);
 
         clDevCtrlStatusFlags_Org = clDevCtrlStatusFlags;
@@ -479,7 +487,7 @@ krcCheckBusError_KERNEL
         if (clDevCtrlStatusFlags &
             NV2080_CTRL_BUS_INFO_PCIE_LINK_ERRORS_CORR_ERROR)
         {
-            NV_PRINTF(LEVEL_ERROR, "     _CORR_ERROR_DETECTED\n");
+            NV_PRINTF(LEVEL_INFO, "PCI-E corelogic: CORR_ERROR_DETECTED\n");
             // not much interested in this one
             clDevCtrlStatusFlags &=
                 ~NV2080_CTRL_BUS_INFO_PCIE_LINK_ERRORS_CORR_ERROR;
@@ -487,63 +495,64 @@ krcCheckBusError_KERNEL
         if (clDevCtrlStatusFlags &
             NV2080_CTRL_BUS_INFO_PCIE_LINK_ERRORS_NON_FATAL_ERROR)
         {
-            NV_PRINTF(LEVEL_ERROR, "     _NON_FATAL_ERROR_DETECTED\n");
+            NV_PRINTF(LEVEL_INFO, "PCI-E corelogic: NON_FATAL_ERROR_DETECTED\n");
         }
         if (clDevCtrlStatusFlags &
             NV2080_CTRL_BUS_INFO_PCIE_LINK_ERRORS_FATAL_ERROR)
         {
-            NV_PRINTF(LEVEL_ERROR, "     _FATAL_ERROR_DETECTED\n");
+            NV_PRINTF(LEVEL_ERROR, "PCI-E corelogic: FATAL_ERROR_DETECTED\n");
         }
         if (clDevCtrlStatusFlags &
             NV2080_CTRL_BUS_INFO_PCIE_LINK_ERRORS_UNSUPP_REQUEST)
         {
-            NV_PRINTF(LEVEL_ERROR, "     _UNSUPP_REQUEST_DETECTED\n");
+            NV_PRINTF(LEVEL_INFO, "PCI-E corelogic: UNSUPP_REQUEST_DETECTED\n");
         }
     }
 
     // Corelogic AER
     if (pCl != NULL && clPcieReadAerCapability(pGpu, pCl, &clAer) == NV_OK &&
-        (clAer.UncorrErrStatusReg != 0 || clAer.RooErrStatus != 0))
+        (clAer.UncorrErrStatusReg != 0 || 
+         (clAer.RooErrStatus & ~CL_AER_ROOT_ERROR_STATUS_ERR_COR_SUBCLASS_MASK) != 0))
     {
-        NV_PRINTF(LEVEL_ERROR,
-                  "PCE-I Advanced Error Reporting Corelogic Info:\n");
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
+                  "PCI-E Advanced Error Reporting Corelogic Info:\n");
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Uncorr Error Status Register    : %08X\n",
                   clAer.UncorrErrStatusReg);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Uncorr Error Mask Register      : %08X\n",
                   clAer.UncorrErrMaskReg);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Uncorr Error Severity Register  : %08X\n",
                   clAer.UncorrErrSeverityReg);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Corr Error Status Register      : %08X\n",
                   clAer.CorrErrStatusReg);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Corr Error Mask Register        : %08X\n",
                   clAer.CorrErrMaskReg);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Advanced Err Cap & Ctrl Register: %08X\n",
                   clAer.AEcapCrtlReg);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Header Log [0-3]                : %08X\n",
                   clAer.HeaderLogReg.Header[0]);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Header Log [4-7]                : %08X\n",
                   clAer.HeaderLogReg.Header[1]);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Header Log [8-B]                : %08X\n",
                   clAer.HeaderLogReg.Header[2]);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Header Log [C-F]                : %08X\n",
                   clAer.HeaderLogReg.Header[3]);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Root Error Command Register     : %08X\n",
                   clAer.RootErrCmd);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Root Error Status               : %08X\n",
                   clAer.RooErrStatus);
-        NV_PRINTF(LEVEL_ERROR,
+        NV_PRINTF(LEVEL_NOTICE,
                   "     Error Source ID Register        : %08X\n",
                   clAer.ErrSrcReg);
 
@@ -585,7 +594,7 @@ _krcValidateAndDumpToKernelLog(NvU64 *lastXidTimestamp)
 {
     NvU32 sec, usec;
 
-    if (((osGetCurrentTime(&sec, &usec) == NV_OK) &&
+    if (((osGetSystemTime(&sec, &usec) == NV_OK) &&
        ((((sec * 1000000) + usec) - *lastXidTimestamp) > 1000000)))
     {
         nvlogDumpToKernelLog(NV_TRUE);
